module GrapeCherry
  module Generators
    class ApiGenerator < Rails::Generators::Base
      source_root File.expand_path('../templates/api', __FILE__)

      argument       :resource_name, required: true

      class_option   :format, aliases: '-f', type: :string, desc: 'The format the API will use. Valid options include "XML" and "JSON"', default: 'json'
      class_option   :api_version, aliases: '-v', type: :string, desc: 'The version of API this endpoint is being generated for.', default: 'v1'
      class_option   :resource_class, aliases: '-c', type: :string, desc: 'The class of the resource if different than the name of the resource'

      # def echo_constant
      #   puts "Type: #{type.to_s}"
      # end


      def generate_api_base
        # if File.exist? 'app/controllers/api/base.rb'
        #
        # else
        #   template 'base.rb', 'app/controllers/api/base.rb'
        # end
      end

      def generate_api_version

      end

      def generate_api_endpoint
        template 'resource.rb', file_path
        template 'helpers.rb', helpers_path
      end


      private
      def valid_format
        unless options.format.downcase == 'xml' or options.format.downcase == 'json'
          options.format = 'json'
        end
        options.format.downcase
      end

      def content_type
        "content_type :#{valid_format}, 'application/#{valid_format}'"
      end

      def version
        options.api_version
      end

      def resource
        resource_name.downcase.pluralize.camelize
      end

      def namespace
        "API::#{version.upcase}::#{resource}"
      end

      def helpers_namespace
        "API::#{version.upcase}::Helpers::#{resource_name.camelize}Helpers"
      end

      def file_path
        "app/controllers/api/#{version}/#{resource.underscore}.rb"
      end

      def helpers_path
        "app/controllers/api/#{version}/helpers/#{resource_name.underscore}_helpers.rb"
      end

      def type_name
        if options.resource_class.present?
          options.resource_class.camelize
        else
          resource_name.camelize
        end
      end

      def type
        if options.resource_class.present?
          options.resource_class.constantize
        else
          resource_name.constantize
        end
      end

      def type_hash
        {
            :string => String,
            :decimal => BigDecimal,
            :integer => Integer,
            :datetime => DateTime,
            :date => Date
        }
      end

      def attribute_required?(attribute)
        type.validators_on(attribute).map(&:class).include?(ActiveRecord::Validations::PresenceValidator)
      end

      def attribute_type(attribute)
        attr_type = type.columns_hash[attribute.to_s].type
        if type_hash.include? attr_type
          type_hash[attr_type].to_s
        else
          String.to_s
        end
      end

      def attributes
        type.attribute_names.map {|attribute| attribute.downcase.to_sym }
      end

    end
  end
end

